/*
m_sumvec_a - Sum a vector of audio signals into one output

DESCRIPTION
This is a summing mixer for a vector of audio signals. A second vector of
static or control signals defines the balance of the inputs.

SYNTAX
aOut m_sumvec_k aInputs[], kBalances[]

INITIALIZATION

PERFORMANCE
aOut - Mix output
aInputs[] - Vector/array of the input signals
kBalances[] - Vector or mix volumes or balance for each input signal

CREDITS
Author: Jeanette C.
*/

opcode m_sumvec_a, a, a[]k[]
	aInputs[], kBalances[] xin

	; Set internal parameters
	iLength = lenarray(aInputs)
	kError init 0 ; internal error flag
	aOut init 0 ; Output signal
	kIndex init 0 ; array index

	; Set an error if the balance array has less elements than the inputs
	if ((lenarray(kBalances) < iLength) && (kError == 0)) then
		kError = 1
		printks("\nERROR: m_sumvec_k balance vector has less elements than input vector.", 1/kr)
		aOut = 0
	endif

	; Only process if there is no previous error
	if (kError == 0) then
		kIndex = 0
		aOut = 0
		while (kIndex < iLength) do
			aOut += aInputs[kIndex] * kBalances[kIndex]
			kIndex += 1
		od
	endif

	; Audio output
	xout(aOut)
endop
