/*
m_sumvec_k - Sum a vector of control signals into one output

DESCRIPTION
This is a summing mixer for a vector of control signals. A second vector of
static or control signals defines the balance of the inputs.

SYNTAX
kOut m_sumvec_k kInputs[], kBalances[]

INITIALIZATION

PERFORMANCE
kOut - Mix output
kInputs[] - Vector/array of the input signals
kBalances[] - Vector or mix volumes or balance for each input signal

CREDITS
Author: Jeanette C.
*/

opcode m_sumvec_k, k, k[]k[]
	kInputs[], kBalances[] xin

	; Set internal parameters
	iLength = lenarray(kInputs)
	kError init 0 ; internal error flag
	kOut init 0 ; Output signal
	kIndex init 0 ; array index

	; Set an error if the balance array has less elements than the inputs
	if ((lenarray(kBalances) < iLength) && (kError == 0)) then
		kError = 1
		printks("\nERROR: m_sumvec_k balance vector has less elements than input vector.", 1/kr)
		kOut = 0
	endif

	; Only process if there is no previous error
	if (kError == 0) then
		kIndex = 0
		kOut = 0
		while (kIndex < iLength) do
			kOut += kInputs[kIndex] * kBalances[kIndex]
			kIndex += 1
		od
	endif

	; Control output
	xout(kOut)
endop
