; Demonstration of the UDOs m_mixvec_k and m_mixvec_a
; these opcodes are sequential switches with a blending/morphing option for
; control (k-rate) and audio (a-rate) signals.
; Default samplerate (sr) is 48000.
<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

#include "m_mixvec_k.udo"
#include "m_mixvec_a.udo"

; Test the control signal switch
; p4 = iMode, p5 = iWrap, p6 = iBlend
instr Control
	; Initialise the parameters
	iMode init p4 ; Indexing mode
	iWrap init p5 ; Wrap around mode
	iBlend init p6 ; Blend mode

	; Set up three different control signals in one array
	kInputs[] init 3
	kInputs[0] = oscil(20, 3, -1) ; sine LFO
	kInputs[1] = randh(20, 2) ; sample&hold noise
	iSquare = ftgenonce(0, 0, 4096, 7, 1, 2048, 1, 0, 0, 2048, 0) ; a square shape
	kInputs[2] = oscil(20, 4, iSquare) ; square wave LFO

	; Set a mix control
	kMix = line(0, p3, 3)
	kOut = m_mixvec_k(kInputs, kMix, iMode, iWrap, iBlend)

	; Set up and oscillator and modulate its frequency with the output from
	; m_mixvec_k
	aOsc = vco2(.5, (110 + kOut), 0)

	; Filter the oscillator a bit and output
	aFilter = butterlp(aOsc, 2000)
	outs(aFilter, aFilter)
endin

; Demonstrate the audio version
; p4 = iMode, p5 = iWrap, p6 = iBlend
instr Audio
	; Initialise the parameters
	iMode init p4 ; Indexing mode
	iWrap init p5 ; Wrap around mode
	iBlend init p6 ; Blend mode

	; Set up a mix control
	kMix = line(0, p3, 3)

	; Create three different audio inputs in an array
	aInputs[] init 3
	aInputs[0] = vco2(.3, 110, 0) ; sawtooth oscillator
	aInputs[2] = noise(.5, 0) ; simple white noise
	aInputs[1] = wguide1(aInputs[2], 200, 3000, .95)

	aOut = m_mixvec_a(aInputs, kMix, iMode, iWrap, iBlend)

	; Output the audio
	outs(aOut, aOut)
endin

</CsInstruments>
; ==============================================
<CsScore>
; Test the control switch/blender
; p4 = iMode, p5 = iWrap, p6 = iBlend
; Real indexing, no wrap, discrete switching
i"Control" 0 6 0 0 0
; real index,ing, no wrap, blending
i. + . 0 0 1
; Normalised index, wrap around, no blend
i. . . 1 1 0
; normalised indexing, wrap around, blending
i. . . 1 1 1

; Test the audio version
; Real indexing, no wrap, discrete switching
i"Audio" 24 6 0 0 0
; Real indexing, no wrap, blend
i. + . 0 0 1
; Normalised indexing, wrap around, switching
i. . . 1 1 0
; Normalised indexing, wrap around, blending
i. . . 1 1 1
e
</CsScore>
</CsoundSynthesizer>
