; Demonstration of the UDOs m_sumvec_k and m_sumvec_a .
; These are summing mixers, which sum an input vector, with balance/volume
; for each element to one output signal.
; The default samplerate (sr) is 48000Hz.
<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

#include "m_sumvec_k.udo"
#include "m_sumvec_a.udo"

; Demonstrate the summing mixer for control signals
instr Control
	; Create an array with three control signals
	kCtls[] init 3
	kCtls[0] = oscil(20, 3, -1) ; a sine LFO
	kCtls[1] = randh(20, 2) ; sample&hold noise
	kCtls[2] = line(200, p3, 0) ; a straight downwards line

	; Create an array with three mix/balance controls, three phase shifted
	; LFOs
	kMix[] init 3
	kMix[0] = oscil(.5, .5, -1, 0) + .5
	kMix[1] = oscil(.5, .5, -1, .25) + .5
	kMix[2] = oscil(.5, .5, -1, .5) + .5

	; Sum the Ctls vector into one output
	kControl = m_sumvec_k(kCtls, kMix)

	; Create an oscillator and modulate its frequency with the summed control
	aOsc = vco2(.5, (110 + kControl), 0)

	; Output the audio
	outs(aOsc, aOsc)
endin

; Demonstrate the summing mixer for an audio vector
instr Audio
	; Create an array with three different audio signals
	aSrc[] init 3
	aSrc[0] = vco2(.3, 110, 0) ; a sawtooth oscillator
	aSrc[1] = vco2(.3, 220, 2, .5)
	aSrc[2] = vco2(.5, 330, 4, .5)

	; Create an array with three mix controls, three phase shifted LFOs
	kMix[] init 3
	kMix[0] = oscil(.5, .5, -1, 0) + .5
	kMix[1] = oscil(.5, .5, -1, .333) + .5
	kMix[2] = oscil(.5, .5, -1, .6667) + .5

	; Sum the audio
	aMix = m_sumvec_a(aSrc, kMix)

	; Output the audio
	outs(aMix, aMix)
endin

</CsInstruments>
; ==============================================
<CsScore>
i"Control" 0 8
i"Audio" 8 8
e
</CsScore>
</CsoundSynthesizer>
