; Demonstration of the UDOs m_vecmix_k and m_vecmix_a . these are vector mixers
; for control (k-rate) and audio (a-rate) signals.
; Default sample rate (sr) is 48000.
<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

#include "m_vecmix_k.udo"
#include "m_vecmix_a.udo"

; p4 = iMode, p5 = iWrap, p6 = iBlend
instr Control
	iMode = p4 ; indexing mode
	iWrap init p5 ; Wrap around mode
	iBlend init p6 ; Blend mode
	; Create a sine LFO
	iSquare = ftgenonce(0, 0, 4096, 7, 1, 2048, 1, 0, 0, 2048, 0)
	kLFO = oscil(10, 3, iSquare)

	; Set up a mix control
	kMix = line(0, p3, 3)

	; Engage the vector mixer
	iNumOutputs = 3 ; three array elements/channels
	kOuts[] = m_vecmix_k(kLFO, kMix, iNumOutputs, iMode, iWrap, iBlend)

	; Create three oscillators, whose frequency is influenced by one vector
	; mixer output each
	aOsc1 = vco2(.2, (110 + kOuts[0]), 0)
	aOsc2 = vco2(.2, (220 + kOuts[1]), 0)
	aOsc3 = vco2(.2, (330 + kOuts[2]), 0)

	; Mix the audio and output
	aMix = aOsc1 + aOsc2 + aOsc3
	outs(aMix, aMix)
endin

; Audio vector mixer
; p4 = iMode, p5 = iWrap, p6 = iBlend
instr Audio
	; Get the inputs
	iMode init p4 ; Indexiing mode
	iWrap init p5  ; wrap around mode
	iBlend init p6 ; blending mode

	; Create a sawtooth oscillator
	aOsc = vco2(.5, 110, 0)

	; Set up a mix control
	kMix = line(0, p3, 3)
	iNumOutputs init 3 ; number of outputs
	aOuts[] = m_vecmix_a(aOsc, kMix, iNumOutputs, iMode, iWrap, iBlend)

	; Pass each output to a different filter
	aLp = butterlp(aOuts[0], 400)
	aHp = butterhp(aOuts[1], 300)
	aBp = butterbp(aOuts[2], 800, 50)

	; Sum the audio and output
	aMix = aLp + aHp + aBp
	outs(aMix, aMix)
endin

</CsInstruments>
; ==============================================
<CsScore>
; Test the control signal vector mixer
; p4 = iMode, p5 = iWrap, p6 = iBlend
; real index, no wrap, discrete switching
i"Control" 0 6 0 0 0
; Real index, no wrap, blending
i. + . 0 0 1
; Normalised index (will go round three times), wrapping, discrete switch
i. . . 1 1 0
; Normalised index, wrap and blend
i. . . 1 1 1

; Test the audio vector mixer
; Real index, no wrap, discrete switching
i"Audio" 24 6 0 0 0
; Real index, no wrap, blending
i. + . 0 0 1
; Normalised index, wrap around, switching
i. . . 1 1 0
; Normalise index, wrap around, blend
i. . . 1 1 1
e
</CsScore>
</CsoundSynthesizer>
