; Dmonestration of the m_vecshift_k and m_vecshift_a UDOs.
; These opcodes sequentially map the inputs of one vector to another with a
; shift/offset position, controllable at k-rate.
; The default samplerate (sr) is 48000Hz.
<CsoundSynthesizer>
<CsOptions>
-o dac -d --messagelevel=2
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	1
nchnls	=	2
0dbfs	=	1

#include "m_vecshift_k.udo"
#include "m_vecshift_a.udo"

; Demonstrate the the vector shifters
; p4 = iMode, p5 = iBlend
instr Shifter
	; Initialise from the p-fields
	iMode init p4 ; indexing mode
	iBlend init p5 ; blend/switch mode

	; Create an array with three input controls
	kInCtls[] init 3
	; A ramp waveform
	iRamp = ftgenonce(0, 0, 4096, 7, 1, 4096, 0)
	kInCtls[0] = oscil(.5, 4, -1) + .5 ; a sine LFO
	kInCtls[1] = oscil(1, 2, iRamp) ; a ramp/saw LFO
	kInCtls[2] = randh(1, 3) ; samle&hold noise

	; Control signal shift position
	kCtlPos = line(5, p3, 0)

	; Create the output control vector
	kOutCtls[] = m_vecshift_k(kInCtls, kCtlPos, iMode, iBlend)

	; Create an input vector of three audio signals, with some aspect controlled
	; by kOutCtls
	aInAudio[] init 3
	aInAudio[0] = vco2(.3, (110 + kOutCtls[0]*20), 4, .5) ; a vco with vibrato
	aInAudio[1] = butterlp(vco2(.2, 220, 0), (200 + kOutCtls[1] * 2000)) ; saw
		; tooth oscillator with varying lowpass filter
	aInAudio[2] = vco2(.2, 330, 2, (kOutCtls[2]*.6 + .2)) ; vco with PWM

	; Position control for audio vector shifter
	kAudioPos = line(0, p3, 3)
	; Shift the audio vector
	aOutAudio[] = m_vecshift_a(aInAudio, kAudioPos, iMode, iBlend)

	; Spread the audio outputs in stereo so the effects are clearly audible
	a1L, a1R pan2 aOutAudio[0], 0
	a2L, a2R pan2 aOutAudio[1], .5
	a3L, a3R pan2 aOutAudio[2], 1
	aOutL = a1L + a2L + a3L
	aOutR = a1R + a2R + a3R

	; Output the audio
	outs(aOutL, aOutR)
endin

</CsInstruments>
; ==============================================
<CsScore>
; Demonstrate the vector shifters
; p4 = iMode, p5 = iBlend
; Raw indexing, no blend
i"Shifter" 0 8 0 0
; Raw indexing, with blend
i. + . 0 1
; Normalised indexing, no blend
i. . . 1 0
; Normalised indexing and blend
i. . . 1 1
e
</CsScore>
</CsoundSynthesizer>
